package org.jayasoft.ivyde.eclipse;

import java.io.File;
import java.net.URL;
import java.util.Collections;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Map;
import java.util.MissingResourceException;
import java.util.ResourceBundle;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jdt.core.JavaModelException;
import org.eclipse.jdt.internal.core.JavaModelManager;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.util.IPropertyChangeListener;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.plugin.AbstractUIPlugin;
import org.jayasoft.ivyde.eclipse.cpcontainer.IvyClasspathContainer;
import org.jayasoft.ivyde.eclipse.ui.console.IvyConsole;
import org.jayasoft.ivyde.eclipse.ui.preferences.PreferenceConstants;
import org.osgi.framework.BundleContext;

import fr.jayasoft.ivy.Ivy;
import fr.jayasoft.ivy.util.Message;

/**
 * The main plugin class to be used in the desktop.
 */
public class IvyPlugin extends AbstractUIPlugin {
	public static final String ID = "org.jayasoft.ivyde.eclipse";
    
    public static final String PREF_CONSOLE_DEBUG_COLOR = ID+ ".console.color.debug";
    public static final String PREF_CONSOLE_VERBOSE_COLOR = ID+ ".console.color.verbose";
    public static final String PREF_CONSOLE_INFO_COLOR = ID+ ".console.color.info";
    public static final String PREF_CONSOLE_WARN_COLOR = ID+ ".console.color.warn";
    public static final String PREF_CONSOLE_ERROR_COLOR = ID+ ".console.color.error";
    
    //The shared instance.
	private static IvyPlugin plugin;
	//Resource bundle.
	private ResourceBundle resourceBundle;
    private IvyConsole console;
    
	
	/**
	 * The constructor.
	 */
	public IvyPlugin() {
		super();
		plugin = this;
	}

	/**
	 * This method is called upon plug-in activation
	 */
	public void start(BundleContext context) throws Exception {
		super.start(context);
        try {
            console = new IvyConsole();
        } catch (RuntimeException e) {
            // Don't let the console bring down the CVS UI
            log(IStatus.ERROR, "Errors occurred starting the Ivy console", e); 
        }
        getPreferenceStore().addPropertyChangeListener(new IPropertyChangeListener() {
            public void propertyChange(PropertyChangeEvent event) {
                if(event.getProperty() == PreferenceConstants.IVYCONF_PATH) {
                    ivyConfPathChanged();
                }
            }
        });
	}

	/**
	 * This method is called when the plug-in is stopped
	 */
	public void stop(BundleContext context) throws Exception {
		super.stop(context);
		plugin = null;
		resourceBundle = null;
//        if (console != null)
//            console.shutdown();
	}

    /**
     * Convenience method for logging statuses to the plugin log
     * 
     * @param status  the status to log
     */
    public static void log(IStatus status) {
        getDefault().getLog().log(status);
    }
    
    public static void log(CoreException e) {
        log(e.getStatus().getSeverity(), "IvyDE internal error", e); 
    }
    
    /**
     * Log the given exception along with the provided message and severity indicator
     */
    public static void log(int severity, String message, Throwable e) {
        log(new Status(severity, ID, 0, message, e));
    }
	/**
	 * Returns the shared instance.
	 */
	public static IvyPlugin getDefault() {
		return plugin;
	}

	/**
	 * Returns the string from the plugin's resource bundle,
	 * or 'key' if not found.
	 */
	public static String getResourceString(String key) {
		ResourceBundle bundle = IvyPlugin.getDefault().getResourceBundle();
		try {
			return (bundle != null) ? bundle.getString(key) : key;
		} catch (MissingResourceException e) {
			return key;
		}
	}

	/**
	 * Returns the plugin's resource bundle,
	 */
	public ResourceBundle getResourceBundle() {
		try {
			if (resourceBundle == null)
				resourceBundle = ResourceBundle.getBundle("org.jayasoft.ivyde.IvyPluginResources");
		} catch (MissingResourceException x) {
			resourceBundle = new ResourceBundle() {
                protected Object handleGetObject(String key) {
                    return null;
                }
            
                public Enumeration getKeys() {
                    return Collections.enumeration(Collections.EMPTY_LIST);
                }
            
            };
		}
		return resourceBundle;
	}

	/**
	 * Returns an image descriptor for the image file at the given
	 * plug-in relative path.
	 *
	 * @param path the path
	 * @return the image descriptor
	 */
	public static ImageDescriptor getImageDescriptor(String path) {
		return AbstractUIPlugin.imageDescriptorFromPlugin("org.jayasoft.ivyde", path);
	}


    private static class IvyConfig {
        Ivy ivy;
        long configTime = -1;
        public IvyConfig() {
        }
        public IvyConfig(Ivy ivy) {
            this.ivy = ivy;
        }
    }
    private static Map _ivysByProject = new HashMap(); // Map (IJavaProject -> IvyConfig)
    private static Map _ivysByConf = new HashMap(); // Map (configPath -> IvyConfig)
    


    public static Ivy getIvy(IJavaProject javaProject) {
        Ivy ivy = refreshIvyConfiguration(javaProject, getIvyconfURL(javaProject));
        return ivy == null ? new Ivy() : ivy;
    }
    
    public static void ivyConfPathChanged() {
        try {
            IJavaProject[] projects = JavaModelManager.getJavaModelManager().getJavaModel().getJavaProjects();
            String defaultConfURL = getIvyconfURL();
            for (int i = 0; i < projects.length; i++) {
                if (getStrictIvyconfURL(projects[i]) == null) {
                    IvyClasspathContainer.resolve(projects[i]);
                }
            }
        } catch (JavaModelException e) {
        }
    }

    private static synchronized Ivy refreshIvyConfiguration(IJavaProject javaProject, String configPath) {        
        try {
            if (configPath == null || configPath.trim().length() == 0 || "default".equals(configPath)) {
                return defaultIvyConfigure(javaProject);
            } else {
                IvyConfig ic = (IvyConfig)_ivysByProject.get(javaProject);
                if (ic == null) {
                    ic = (IvyConfig)_ivysByConf.get(configPath);
                    if (ic == null) {
                        ic = new IvyConfig();
                        _ivysByProject.put(javaProject, ic);
                        _ivysByConf.put(configPath, ic);
                    }
                }
                
                URL url = new URL(configPath);
                if (url.getProtocol().startsWith("file")) {
                    File file = new File(url.getPath());
                    if (!file.exists()) {
                        MessageDialog.openWarning(PlatformUI.getWorkbench().getActiveWorkbenchWindow().getShell(), "No ivyConf found", configPath+" ivyconf cannot be found.\nPlease set your ivy conf url in the preference or in your project properties to be able to use IvyDE");
                        if (ic.ivy == null) {
                            ic.ivy = new Ivy();
                        }
                    } else {
                        if (file.lastModified() != ic.configTime) {
                            ic.ivy = new Ivy();
                            if (ic.configTime == -1) {
                                Message.info("\n\n");
                            } else {
                                Message.info("\n\nIVYDE: ivyconf has changed, configuring ivy again\n");
                            }
                            ic.ivy.configure(file);
                            ic.configTime = file.lastModified();
                        }
                    }
                } else {
                    if (ic.ivy == null) {
                        ic.ivy = new Ivy();
                        ic.ivy.configure(url);
                    }
                }
                return ic.ivy;
            }
        } catch (Exception e) {
            try {
                MessageDialog.openWarning(PlatformUI.getWorkbench().getActiveWorkbenchWindow().getShell(), "Bad ivyConf found", "Problem occured while using "+configPath+" to configure Ivy.\nPlease set your ivy conf url properly in the preference or in the project properties to be able to use IvyDE.\nException message: "+e.getMessage());
            } catch (Exception ex) {}
            log(IStatus.WARNING, "Problem occured while using "+configPath+" to configure Ivy", e);
            Message.warn("IVYDE: Problem occured while using "+configPath+" to configure Ivy. See error log for details");
            _ivysByProject.remove(javaProject);
            _ivysByConf.remove(configPath);
            return new Ivy();
        }
    }

    private static Ivy defaultIvyConfigure(IJavaProject javaProject) {
        IvyConfig ic = (IvyConfig)_ivysByProject.get(javaProject);
        if (ic == null) {
            ic = (IvyConfig)_ivysByConf.get("default");
        }
        Ivy ivy = ic == null ? null : ic.ivy;
        if (ivy == null) {
            ivy = new Ivy();
            try {
                ivy.configureDefault();
                IvyConfig ivyConfig = new IvyConfig(ivy);
                _ivysByProject.put(javaProject, ivyConfig);
                _ivysByConf.put("default", ivyConfig);
            } catch (Exception ex) {
                MessageDialog.openWarning(PlatformUI.getWorkbench().getActiveWorkbenchWindow().getShell(), "Impossible to configure Ivy", "Problem occured while configuring Ivy with its default configuration.\nPlease set an ivy conf url properly in the preference or in the project properties to be able to use IvyDE.\nException message: "+ex.getMessage());
                log(IStatus.WARNING, "Problem occured while configuring Ivy with its default configuration.", ex);
                Message.warn("IVYDE: Problem occured while configuring Ivy with its default configuration.. See error log for details");
            }
        }
        return ivy;
    }

    public static String getIvyconfURL(IJavaProject project) {
        String ivyconf = getStrictIvyconfURL(project);
        if (ivyconf == null) {
            return getIvyconfURL();
        } else {
            return ivyconf;
        }
    }

    public static String getStrictIvyconfURL(IJavaProject project) {
        if (project == null) {
            return null;
        }
        String opt = IvyPlugin.getDefault().getPreferenceStore().getString(PreferenceConstants.IVYCONF_PATH+"."+project.getPath());
        if (opt == null || opt.trim().length() == 0 || "inherited".equals(opt)) {
            return null;
        } else {
            return opt.trim();
        }
    }

    public static String getIvyconfURL() {
        String configPath = IvyPlugin.getDefault().getPreferenceStore().getString(PreferenceConstants.IVYCONF_PATH);
        if (configPath == null || configPath.trim().length() == 0) {
            return "default";
        } else {
            return configPath.trim();
        }
    }

    public static void setIvyconfURL(IJavaProject project, String ivyconfURL) {
        _ivysByProject.remove(project);
        if (ivyconfURL == null) {
            IvyPlugin.getDefault().getPreferenceStore().setValue(PreferenceConstants.IVYCONF_PATH+"."+project.getPath(), "inherited");
        } else {
            IvyPlugin.getDefault().getPreferenceStore().setValue(PreferenceConstants.IVYCONF_PATH+"."+project.getPath(), ivyconfURL);
        }
        
        IvyClasspathContainer.resolve(project);
    }

    public IvyConsole getConsole() {
        return console;
    }
}
